@@ -34,12 +34,8 @@ Button::Button(DecorationButtonType type, Decoration *decoration, QObject *paren
         setOpacity(value.toReal());
     });
 
-    // setup default geometry
-    const int height = decoration->buttonHeight();
-    setGeometry(QRect(0, 0, height, height));
-    setIconSize(QSize(height, height));
-
     // connections
+    connect(decoration, &Decoration::tabletModeChanged, this, &Button::reconfigure);
     connect(decoration->client(), SIGNAL(iconChanged(QIcon)), this, SLOT(update()));
     connect(decoration->settings().get(), &KDecoration2::DecorationSettings::reconfigured, this, &Button::reconfigure);
     connect(this, &KDecoration2::DecorationButton::hoveredChanged, this, &Button::updateAnimationState);
@@ -51,10 +47,7 @@ Button::Button(DecorationButtonType type, Decoration *decoration, QObject *paren
 Button::Button(QObject *parent, const QVariantList &args)
     : Button(args.at(0).value<DecorationButtonType>(), args.at(1).value<Decoration *>(), parent)
 {
-    m_flag = FlagStandalone;
-    //! icon size must return to !valid because it was altered from the default constructor,
-    //! in Standalone mode the button is not using the decoration metrics but its geometry
-    m_iconSize = QSize(-1, -1);
+    setGeometry(QRectF(QPointF(0, 0), preferredSize()));
 }
 
 //__________________________________________________________________
@@ -114,22 +107,9 @@ void Button::paint(QPainter *painter, const QRect &repaintRegion)
         return;
     }
 
-    painter->save();
-
-    // translate from offset
-    if (m_flag == FlagFirstInList) {
-        painter->translate(m_offset);
-    } else {
-        painter->translate(0, m_offset.y());
-    }
-
-    if (!m_iconSize.isValid() || isStandAlone()) {
-        m_iconSize = geometry().size().toSize();
-    }
-
-    // menu button
-    if (type() == DecorationButtonType::Menu) {
-        const QRectF iconRect(geometry().topLeft(), m_iconSize);
+    switch (type()) {
+    case KDecoration2::DecorationButtonType::Menu: {
+        const QRectF iconRect = geometry().marginsRemoved(m_padding);
         const auto c = decoration()->client();
         if (auto deco = qobject_cast<Decoration *>(decoration())) {
             const QPalette activePalette = KIconLoader::global()->customPalette();
@@ -145,12 +125,16 @@ void Button::paint(QPainter *painter, const QRect &repaintRegion)
         } else {
             c->icon().paint(painter, iconRect.toRect());
         }
-
-    } else {
+        break;
+    }
+    case KDecoration2::DecorationButtonType::Spacer:
+        break;
+    default:
+        painter->save();
         drawIcon(painter);
+        painter->restore();
+        break;
     }
-
-    painter->restore();
 }
 
 //__________________________________________________________________
@@ -163,9 +147,10 @@ void Button::drawIcon(QPainter *painter) const
     this makes all further rendering and scaling simpler
     all further rendering is performed inside QRect( 0, 0, 18, 18 )
     */
-    painter->translate(geometry().topLeft());
+    const QRectF rect = geometry().marginsRemoved(m_padding);
+    painter->translate(rect.topLeft());
 
-    const qreal width(m_iconSize.width());
+    const qreal width(rect.width());
     painter->scale(width / 20, width / 20);
     painter->translate(1, 1);
 
@@ -383,9 +368,20 @@ void Button::reconfigure()
 {
     // animation
     auto d = qobject_cast<Decoration *>(decoration());
-    if (d) {
-        m_animation->setDuration(d->animationsDuration());
+    if (!d) {
+        return;
     }
+
+    switch (type()) {
+    case KDecoration2::DecorationButtonType::Spacer:
+        setPreferredSize(QSizeF(d->buttonSize() * 0.5, d->buttonSize()));
+        break;
+    default:
+        setPreferredSize(QSizeF(d->buttonSize(), d->buttonSize()));
+        break;
+    }
+
+    m_animation->setDuration(d->animationsDuration());
 }
 
 //__________________________________________________________________
