@@ -19,7 +19,6 @@
 #endif
 
 #include <QApplication>
-#include <QDBusConnection>
 #include <QDockWidget>
 #include <QFileInfo>
 #include <QMainWindow>
@@ -69,8 +68,8 @@ bool PaletteChangedEventFilter::eventFilter(QObject *watched, QEvent *event)
 }
 
 //____________________________________________________________________
-Helper::Helper(KSharedConfig::Ptr config, QObject *parent)
-    : QObject(parent)
+Helper::Helper(KSharedConfig::Ptr config)
+    : QObject()
     , _config(std::move(config))
     , _kwinConfig(KSharedConfig::openConfig("kwinrc"))
     , _decorationConfig(new InternalSettings())
@@ -585,7 +584,8 @@ void Helper::renderMenuFrame(QPainter *painter, const QRectF &rect, const QColor
     if (roundCorners) {
         painter->setRenderHint(QPainter::Antialiasing);
         QRectF frameRect(rect);
-        qreal radius(frameRadius(PenWidth::NoPen));
+
+        qreal radius(Metrics::Frame_FrameRadius);
 
         frameRect.adjust( //
             seamlessEdges.testFlag(Qt::LeftEdge) ? -radius : 0,
@@ -630,6 +630,39 @@ void Helper::renderMenuFrame(QPainter *painter, const QRectF &rect, const QColor
     painter->restore();
 }
 
+QRegion Helper::menuFrameRegion(const QMenu *widget)
+{
+    if (!widget) {
+        return {};
+    }
+
+    const bool hasAlpha(hasAlphaChannel(widget));
+    const auto seamlessEdges = menuSeamlessEdges(widget);
+    const auto roundCorners = hasAlpha;
+
+    if (roundCorners) {
+        QRectF frameRect(widget->rect());
+
+        qreal radius(Metrics::Frame_FrameRadius);
+
+        frameRect.adjust( //
+            seamlessEdges.testFlag(Qt::LeftEdge) ? -radius : 0,
+            seamlessEdges.testFlag(Qt::TopEdge) ? -radius : 0,
+            seamlessEdges.testFlag(Qt::RightEdge) ? radius : 0,
+            seamlessEdges.testFlag(Qt::BottomEdge) ? radius : 0);
+
+        // outline is always valid/drawn
+        frameRect = strokedRect(frameRect);
+        radius = frameRadiusForNewPenWidth(radius, PenWidth::Frame);
+
+        QPainterPath path;
+        path.addRoundedRect(frameRect, radius, radius);
+        return QRegion(path.toFillPolygon().toPolygon()).intersected(widget->rect());
+    }
+
+    return QRegion(widget->rect());
+}
+
 //______________________________________________________________________________
 void Helper::renderButtonFrame(QPainter *painter,
                                const QRectF &rect,
@@ -853,7 +886,7 @@ void Helper::renderCheckBoxBackground(QPainter *painter,
 
     switch (state) {
     case CheckOff:
-        painter->setBrush(palette.base().color().darker(sunken ? radioCheckSunkenDarkeningFactor : 100));
+        painter->setBrush(palette.button().color().darker(sunken ? radioCheckSunkenDarkeningFactor : 100));
         painter->drawRoundedRect(frameRect, radius, radius);
         break;
 
@@ -864,7 +897,7 @@ void Helper::renderCheckBoxBackground(QPainter *painter,
         break;
 
     case CheckAnimated:
-        painter->setBrush(palette.base().color().darker(sunken ? radioCheckSunkenDarkeningFactor : 100));
+        painter->setBrush(palette.button().color().darker(sunken ? radioCheckSunkenDarkeningFactor : 100));
         painter->drawRoundedRect(frameRect, radius, radius);
         painter->setBrush(transparent);
         painter->setOpacity(animation);
@@ -1011,7 +1044,7 @@ void Helper::renderRadioButtonBackground(QPainter *painter,
 
     switch (state) {
     case RadioOff:
-        painter->setBrush(palette.base().color().darker(sunken ? radioCheckSunkenDarkeningFactor : 100));
+        painter->setBrush(palette.button().color().darker(sunken ? radioCheckSunkenDarkeningFactor : 100));
         painter->drawEllipse(frameRect);
         break;
     case RadioOn:
@@ -1019,7 +1052,7 @@ void Helper::renderRadioButtonBackground(QPainter *painter,
         painter->drawEllipse(frameRect);
         break;
     case RadioAnimated:
-        painter->setBrush(palette.base().color().darker(sunken ? radioCheckSunkenDarkeningFactor : 100));
+        painter->setBrush(palette.button().color().darker(sunken ? radioCheckSunkenDarkeningFactor : 100));
         painter->drawEllipse(frameRect);
         painter->setBrush(transparent);
         painter->setOpacity(animation);
@@ -1332,7 +1365,7 @@ void Helper::renderTabBarTab(QPainter *painter,
 
     if (selected) {
         // overlap border
-        // This covers just enough of the border, so that both the border and it's
+        // This covers just enough of the border, so that both the border and its
         // antialiasing effect is covered. On 100% scale it does nothing
         const qreal overlap = devicePixelRatio(painter) * devicePixelRatio(painter);
         frameRect.adjust(east ? -overlap : 0, south ? -overlap : 0, west ? overlap : 0, north ? overlap : 0);
@@ -1347,9 +1380,9 @@ void Helper::renderTabBarTab(QPainter *painter,
         painter->setPen(QPen(penBrush, PenWidth::Frame));
         QRectF highlightRect = frameRect;
         if (north || south) {
-            highlightRect.setHeight(Metrics::Frame_FrameRadius);
+            highlightRect.setHeight(Metrics::TabBar_ActiveEffectSize);
         } else if (west || east) {
-            highlightRect.setWidth(Metrics::Frame_FrameRadius);
+            highlightRect.setWidth(Metrics::TabBar_ActiveEffectSize);
         }
         if (south) {
             highlightRect.moveBottom(frameRect.bottom());
@@ -1364,7 +1397,7 @@ void Helper::renderTabBarTab(QPainter *painter,
         painter->drawPath(highlightPath);
     } else {
         // don't overlap border
-        // Since we dont set the rectangle as strokedRect here, modify only one side of it
+        // Since we don't set the rectangle as strokedRect here, modify only one side of it
         // the same amount strokedRect method would, to make it snap next to the border
         const qreal overlap = PenWidth::Frame;
         frameRect.adjust(east ? overlap : 0, south ? overlap : 0, west ? -overlap : 0, north ? -overlap : 0);
